--[[
    Aaron MacDonald
    August 28, 2010
]]

require('graphics')
require('constants')
require('utility')

local SPRITE_NORTH = '_n'
local SPRITE_EAST = '_e'
local SPRITE_SOUTH = '_s'
local SPRITE_WEST = '_w'

actor = {}

function actor.diagonalSpeed(speed)
    return utility.round((speed * speed)
      / math.sqrt((speed * speed) * 2))
end

function actor.newActor(actorType, x, y, facing)
    local spriteSheet = graphics.loadImage(actorType.spriteSheet)
    local sheetWidth = spriteSheet:getWidth()
    local sheetHeight = spriteSheet:getHeight()

    local actWidth = actorType.spriteWidth
    local actHeight = actorType.spriteHeight

    local spriteQuads = {
      [constants.UP] = love.graphics.newQuad(0, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.RIGHT] = love.graphics.newQuad( actWidth, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.DOWN] = love.graphics.newQuad(actWidth * 2, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.LEFT] = love.graphics.newQuad(actWidth * 3, 0,
        actWidth, actHeight, sheetWidth, sheetHeight)
    }

    local act = {
        type = actorType,
        spriteSheet = spriteSheet,
        spriteQuads = spriteQuads,
        x = x,
        y = y,
        dx = 0,
        dy = 0,
        facing = facing
    }

    return act
end

function actor.updateVelocity(act, movementControls)
    act.dx = 0
    act.dy = 0

    local speed = act.type.speed
    local diagonalSpeed = act.type.diagonalSpeed

    if movementControls[constants.UP] then
        act.dy = act.dy - speed
    end

    if movementControls[constants.DOWN] then
        act.dy = act.dy + speed
    end

    if movementControls[constants.RIGHT] then
        act.dx = act.dx + speed
    end

    if movementControls[constants.LEFT] then
        act.dx = act.dx - speed
    end

    if act.dx == 0 then
        if act.dy > 0 then
            act.facing = constants.DOWN
        elseif act.dy < 0 then
            act.facing = constants.UP
        end
    end

    if act.dy == 0 then
        if act.dx > 0 then
            act.facing = constants.RIGHT
        elseif act.dx < 0 then
            act.facing = constants.LEFT
        end
    end

    if (act.dx ~= 0) and (act.dy ~= 0) then
        act.dx = diagonalSpeed * utility.sign(act.dx)
        act.dy = diagonalSpeed * utility.sign(act.dy)
    end
end

function actor.isMoving(act)
    return (act.dx ~= 0) or (act.dy ~= 0)
end

function actor.move(act, dx, dy)
    act.x = act.x + dx
    act.y = act.y + dy
end

function actor.draw(act)
    love.graphics.drawq(act.spriteSheet, act.spriteQuads[act.facing],
      act.x - act.type.spriteOffsetX, act.y - act.type.spriteOffsetY)
end